clear;
clc;

N = 15;

% THERMOSTAT - modeling and open loop prediction
% x - temperature
% u - heating switch - unused in this model

names = {'x' 'u' };
varsizes = {[1,1] [1,1] };
indicators = { 'r' 'b' };
param.ineq = { ' [ min_temp <= x{k} <= max_temp ]' };
param.eq = {}; 
param.val = { ' max_temp = 50; min_temp = 0; heat = 1; ' };
variables = yalmip_fsm_variables(names, varsizes, indicators, param );

% s1 - heating off  s2 - heating on
s1 = yalmip_fsm_state('[x{k+1} == x{k} - heat ]');
s2 = yalmip_fsm_state('[x{k+1} == x{k} + heat ]');
states = [ s1, s2 ];
% demonstration of OR realtion
% t12 = yalmip_fsm_transition(s1, s2, '[ x{k} < 21.1 ]', '[ x{k} < 22.1 ]');
t12 = yalmip_fsm_transition(s1, s2, '[ x{k} < 21.1 ]' );
t21 = yalmip_fsm_transition(s2, s1, '[ x{k} > 22.9 ]');
trans = [t12, t21];

% 'unary' - basic approach , 'binary' - binary encoded states
Options.encoding = 'unary';
% cuts: 'basic' - one inf node = one cut, 'enhanced' = complex cuts
Options.cuts = 'basic';
% 0 - silent, 1 - elapsed time
Options.verbose = 1;

[fsm, V] = yalmip_fsm(states, trans , variables, N , Options)

% ------------------optim--------------------------------------
obj = 0;
for k = 1:N
obj = obj+sum(V.t{k});
% obj = obj - V.x{k}(1);
end

options = sdpsettings('savesolverinput',1,'savesolveroutput',1,'saveyalmipmodel',1);
if isequal(Options.encoding,'unary')  
    yal = solvesdp(fsm + [ V.in_state{1} == [1;0], V.x{1} == 21 ],obj,options)
elseif isequal(Options.encoding,'binary')  
    yal = solvesdp(fsm + [ V.d{1} == [0;0], V.x{1} == 21 ],obj,options)
end

% --------------plot----------------------------
figure
subplot(3,1,1)
% tresholds
UB = repmat(23,1,N+1);
LB = repmat(21,1,N+1);
pred_hor = 0:N;
St = double([V.x{:}]);
hold on
plot(pred_hor,St(1,:),'b*-','LineWidth',3)
plot(pred_hor,UB,'r-.','LineWidth',3)
plot(pred_hor,LB,'r--','LineWidth',3)
title('x = temperature')
legend('temp','UB','LB')
grid on
axis([0 N 19 25]) 

subplot(3,1,2)
% states
    if isequal(Options.encoding,'unary')  
       reg = double([V.in_state{:}]);
    elseif isequal(Options.encoding,'binary')  
       reg =  double([V.d{:}]);
    end
hold on
stairs(pred_hor,reg(1,:),'b-','LineWidth',3)
if isequal(Options.encoding,'unary')  
    stairs(pred_hor,reg(2,:),'g--','LineWidth',3)
    legend('S1','S2')
elseif isequal(Options.encoding,'binary') 
    legend('d1')
end
title('States')
axis([0 N -0.1 1.1]) 

subplot(3,1,3)
% trans
tr = double([V.t{:}]);
hold on
stairs(pred_hor(1:end-1),tr(1,:),'b-','LineWidth',3)
stairs(pred_hor(1:end-1),tr(2,:),'g--','LineWidth',3)
legend('t12','t21')
title('trans')
axis([0 N -0.1 1.1]) 









