clear;
clc;

% MODELING AND OPEN LOOP SIMULATION OF A TRUCK MODEL
% unary and binary encoding of states

N = 15;  % prediction horizon
% Declaration of variables
names = {'x' 'u' 'mo'};
varsizes = {[2,1] [3,1] [1,1] };
indicators = { 'r' 'r' 'b'};
%  params eq and ineq could be the same!!! replace for param.constr
param.ineq = { ' [ -0.1*max_speed <= x{k}(1) <= (target_dis + 0.1*max_speed) , 0 <= x{k}(2) <= max_cap]'; 
    '[0 <= u{k}(1)<=max_cap, 0 <= u{k}(2)<=max_cap , u{k}(3)>=0, u{k}(3)<=max_speed]' };
param.eq = {}; 
param.val = { ' max_cap = 100; max_speed = 200; target_dis = 300' };
variables = yalmip_fsm_variables(names, varsizes, indicators, param );
% x = (1)distance, (2)capacity 
% u = (1)load, (2)unload, (3)speed
% mo = move

% Declaration of states
s1 = yalmip_fsm_state('[x{k+1}(2) == x{k}(2)+u{k}(1) , u{k}(2) <= 0 , u{k}(3) <= 0 , x{k+1}(1) == x{k}(1) ]');
s2 = yalmip_fsm_state('[x{k+1}(1) == x{k}(1)+u{k}(3) , u{k}(2) <= 0 , u{k}(1) <= 0 , x{k+1}(2) == x{k}(2) ]');
s3 = yalmip_fsm_state('[x{k+1}(2) == x{k}(2)-u{k}(2) , u{k}(1) <= 0 , u{k}(3) <= 0 , x{k+1}(1) == x{k}(1) ]');
s4 = yalmip_fsm_state('[x{k+1}(1) == x{k}(1)-u{k}(3) , u{k}(2) <= 0 , u{k}(1) <= 0 , x{k+1}(2) == x{k}(2) ]'); 
states = [ s1, s2 , s3, s4];
% Declaration of transitions
t12 = yalmip_fsm_transition(s1, s2, '[ mo{k}(1) > 0.5 ]');
t23 = yalmip_fsm_transition(s2, s3, '[ x{k}(1) >= 300 ]');
% t23 = yalmip_fsm_transition(s2, s3, '[ x{k}(1) >= 300 ]', '[ x{k}(1) >= 400 ]', '[ x{k}(1) >= 600 ]');
t34 = yalmip_fsm_transition(s3, s4, '[ mo{k}(1) > 0.5 ]');
t41 = yalmip_fsm_transition(s4, s1, '[ x{k}(1) <= 0 ]');
trans = [t12, t23, t34, t41];

%----------- MODELING FUNCTION ---------------------------------
% 'unary' - basic approach , 'binary' - binary encoded states
Options.encoding = 'unary';
% cuts: 'basic' - one inf node = one ineq , 'enhanced' = complex cuts
Options.cuts = 'basic';
% 0 - silent, 1 - elapsed time + ( number of MIP constr??? -not yet)
Options.verbose = 1;
[fsm, V] = yalmip_fsm(states, trans , variables, N, Options );

% ------------------ OPEN LOOP PREDICTION ----------------------
% OBJECTIVE FUNCTION
obj = 0;  % inicialization of objective function
for k = 1:N+1
obj = obj - V.u{k}(2);    % maximization of unload
end

% OPTIMIZATION
options = sdpsettings('savesolverinput',1,'savesolveroutput',1,'saveyalmipmodel',1);

% initial conditions: S1 = 1, distance = 0, capacity = 0
if isequal(Options.encoding,'unary')  
    yal = solvesdp(fsm + [V.in_state{1} == [1;0;0;0],V.x{1} == [0;0]],obj,options)
elseif isequal(Options.encoding,'binary')  
    yal = solvesdp(fsm + [V.d{1} ==[ 0; 0 ],V.x{1} == [0;0]],obj,options)
end

% ------------PLOTING------------------------------
pred_hor = 0:N;
figure
% continuous states - x = distance, capacity
St = double([V.x{:}]);
subplot(3,1,1)
hold on
stairs(pred_hor,St(1,:),'b','LineWidth',3)
stairs(pred_hor,St(2,:),'g--','LineWidth',3)
title('x = distance, capacity')
legend('distance','capacity')
grid on
axis([0 N 0 400]) 

% continuous inputs - u = load, unload, speed
In = double([V.u{:}]);
subplot(3,1,2)
hold on
stairs(pred_hor,In(1,:),'b','LineWidth',3)
stairs(pred_hor,In(2,:),'g--','LineWidth',3)
stairs(pred_hor,In(3,:),'r-.','LineWidth',3)
title('u = load, unload, speed')
legend('load','unload','speed')
grid on
% binary input - mo = move
Bi = double([V.mo{:}]);
subplot(3,1,3)
hold on
stairs(pred_hor(1:end-1),Bi(1,:),'b','LineWidth',3)
title('mo = move')
% grid on
axis([0 N -0.1 1.1]) 


% states - regions 
figure
subplot(2,1,1)
    if isequal(Options.encoding,'unary')  
       reg = double([V.in_state{:}]);
    elseif isequal(Options.encoding,'binary')  
       reg =  double([V.d{:}]);
    end
hold on
stairs(pred_hor,reg(1,:),'b','LineWidth',3)
stairs(pred_hor,reg(2,:),'r--','LineWidth',3)
 if isequal(Options.encoding,'unary')  
    stairs(pred_hor,reg(3,:),'g-.','LineWidth',3)
    stairs(pred_hor,reg(4,:),'c:','LineWidth',3)
    legend('S1','S2','S3','S4')
    title('States')
 elseif isequal(Options.encoding,'binary') 
     legend('d1','d2')
     title('states encoding Si=[d1 d2] => S1=[0 0] S2=[0 1] S3=[1 0] S4=[1 1]')
 end
%  grid on
 axis([0 N -0.1 1.1]) 

% trans
subplot(2,1,2)
tr = double([V.t{:}]);
hold on
stairs(pred_hor(1:end-1),tr(1,:),'b','LineWidth',3)
stairs(pred_hor(1:end-1),tr(2,:),'r--','LineWidth',3)
stairs(pred_hor(1:end-1),tr(3,:),'g-.','LineWidth',3)
stairs(pred_hor(1:end-1),tr(4,:),'c:','LineWidth',3)
% grid on
legend('t12','t23','t34','t41')
title('trans')
axis([0 N -0.1 1.1]) 




